# Import necessary libraries
from autogonai.development.base import BaseBlock, gen_params


# Define class AutoImageClassifierInit
class AutoImageClassifierInit(BaseBlock):
    def set_params(self, hyp_params: dict = None):
        """
        Sets parameters for AutoImageClassifierInit class.

        Args:
            hyp_params (dict, optional): Hyperparameters for the classifier. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoImageClassifierTrain
class AutoImageClassifierTrain(BaseBlock):
    def set_params(self, model_name: str = None, hyp_params: dict = None):
        """
        Sets parameters for AutoImageClassifierTrain class.

        Args:
            model_name (str, optional): Name of the classifier model. Defaults to None.
            hyp_params (dict, optional): Hyperparameters for the classifier. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoTextClassifierInit
class AutoTextClassifierInit(BaseBlock):
    def set_params(self, hyp_params: dict = None):
        """
        Sets parameters for AutoTextClassifierInit class.

        Args:
            hyp_params (dict, optional): Hyperparameters for the classifier. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoTextClassifierTrain
class AutoTextClassifierTrain(BaseBlock):
    def set_params(self, model_name: str = None, hyp_params: dict = None):
        """
        Sets parameters for AutoTextClassifierTrain class.

        Args:
            model_name (str, optional): Name of the classifier model. Defaults to None.
            hyp_params (dict, optional): Hyperparameters for the classifier. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoStructuredDataClassifierInit
class AutoStructuredDataClassifierInit(BaseBlock):
    def set_params(self, hyp_params: dict = None):
        """
        Sets parameters for AutoStructuredDataClassifierInit class.

        Args:
            hyp_params (dict, optional): Hyperparameters for the classifier. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoStructuredDataClassifierTrain
class AutoStructuredDataClassifierTrain(BaseBlock):
    def set_params(self, model_name: str = None, hyp_params: dict = None):
        """
        Sets parameters for AutoStructuredDataClassifierTrain class.

        Args:
            model_name (str, optional): Name of the classifier model. Defaults to None.
            hyp_params (dict, optional): Hyperparameters for the classifier. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoImageRegressorInit
class AutoImageRegressorInit(BaseBlock):
    def set_params(self, hyp_params: dict = None):
        """
        Sets parameters for AutoImageRegressorInit class.

        Args:
            hyp_params (dict, optional): Hyperparameters for the regressor. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoImageRegressorTrain
class AutoImageRegressorTrain(BaseBlock):
    def set_params(self, model_name: str = None, hyp_params: dict = None):
        """
        Sets parameters for AutoImageRegressorTrain class.

        Args:
            model_name (str, optional): Name of the regressor model. Defaults to None.
            hyp_params (dict, optional): Hyperparameters for the regressor. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoTextRegressorInit
class AutoTextRegressorInit(BaseBlock):
    def set_params(self, hyp_params: dict = None):
        """
        Sets parameters for AutoTextRegressorInit class.

        Args:
            hyp_params (dict, optional): Hyperparameters for the regressor. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoTextRegressorTrain
class AutoTextRegressorTrain(BaseBlock):
    def set_params(self, model_name: str = None, hyp_params: dict = None):
        """
        Sets parameters for AutoTextRegressorTrain class.

        Args:
            model_name (str, optional): Name of the regressor model. Defaults to None.
            hyp_params (dict, optional): Hyperparameters for the regressor. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoStructuredDataRegressorInit
class AutoStructuredDataRegressorInit(BaseBlock):
    def set_params(self, hyp_params: dict = None):
        """
        Sets parameters for AutoStructuredDataRegressorInit class.

        Args:
            hyp_params (dict, optional): Hyperparameters for the regressor. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoStructuredDataRegressorTrain
class AutoStructuredDataRegressorTrain(BaseBlock):
    def set_params(self, model_name: str = None, hyp_params: dict = None):
        """
        Sets parameters for AutoStructuredDataRegressorTrain class.

        Args:
            model_name (str, optional): Name of the regressor model. Defaults to None.
            hyp_params (dict, optional): Hyperparameters for the regressor. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params



from .api import *
from autogonai.development import Blocks
from autogonai.connector import API


class Client(API):
    """
    The AutogonAI Python Client that acts as an interface between AutogonAI and Python projects

    Args:
        api_key (str): The API key gotten from user settings from the AutogonAI platform

    """

    def __init__(self, api_key, **kwargs):
        self.api_key = api_key

        # TODO: some functionality to validate user's api_key

        super().__init__(api_key=api_key, **kwargs)

        self.Projects = Projects(self)
        self.StateManagements = StateManagements(self)
        self.Datasets = Datasets(self)
        self.Blocks = Blocks(self)
        self.Production = Production(self)
        self.Qore = Qore(self)



class function_codes:
    # Data Processing
    DataInput = "DP_1"
    HandleMissingData = "DP_2"
    DataEncoding = "DP_3"
    DataSplitting = "DP_4"
    FeatureScaleData = "DP_5"
    DropColumns = "DP_6"  # "DP_DC"
    TimeStepData = "DP_7"  # "DP_TSP"

    AutomatedDataProcessing = "DP_ADP"
    FeatureSampling = "DP_FSP"
    ParseDatetime = "DP_PDT"
    ReorderColumn = "DP_ROC"
    ReshapeArray = "DP_RSH"
    ColumnAstype = "DP_ASP"
    ShowDuplicates = "DP_SDC"
    DropDuplicates = "DP_DRD"
    ScalarToNdarray = "DP_STN"
    ImageToNdarray = "DP_ITN"
    DatasetInfo = "DP_INF"
    DatasetCorrelations = "DP_CRR"
    DatasetDescription = "DP_DSC"
    DatasetUniques = "DP_UNQ"
    StatsCounts = "DP_STC"
    PrincipalComponentAnalysis = "DP_PCA"
    TextVectorizer = "DP_VEC"
    Resampler = "DP_RES"

    # Data Visualization
    ScatterPlot = "DP_SCP"
    OrdinaryPlot = "DP_ORD"
    CompareScatterPlots = "DP_CSP"
    PiePlot = "DP_PIE"
    HeatmapPlot = "DP_HMP"

    # Machine Learning
    SimpleLinearRegression = "ML_R_1"
    SimpleLinearRegressionPredict = "ML_R_1_P"
    SimpleLinearRegressionPredict = "ML_R_1_M"
    MultipleLinearRegression = "ML_R_2"
    MultipleLinearRegressionPredict = "ML_R_2_P"
    MultipleLinearRegressionPredict = "ML_R_2_M"
    PolynomialLinearRegression = "ML_R_3"
    PolynomialLinearRegressionPredict = "ML_R_3_P"
    PolynomialLinearRegressionPredict = "ML_R_3_M"
    SupportVectorRegression = "ML_R_4"
    SupportVectorRegressionPredict = "ML_R_4_P"
    SupportVectorRegressionPredict = "ML_R_4_M"
    DecisionTreeRegression = "ML_R_5"
    DecisionTreeRegressionPredict = "ML_R_5_P"
    DecisionTreeRegressionPredict = "ML_R_5_M"
    RandomForestRegression = "ML_R_6"
    RandomForestRegressionPredict = "ML_R_6_P"
    RandomForestRegressionPredict = "ML_R_6_M"
    LogisticRegression = "ML_CN_1"
    LogisticRegressionPredict = "ML_CN_1_P"
    LogisticRegressionPredict = "ML_CN_1_M"
    KNearestNeighbors = "ML_CN_2"
    KNearestNeighborsPredict = "ML_CN_2_P"
    KNearestNeighborsPredict = "ML_CN_2_M"
    SupportVectorMachine = "ML_CN_3"
    SupportVectorMachinePredict = "ML_CN_3_P"
    SupportVectorMachinePredict = "ML_CN_3_M"
    KernelSupportVectorMachine = "ML_CN_4"
    KernelSupportVectorMachinePredict = "ML_CN_4_P"
    KernelSupportVectorMachinePredict = "ML_CN_4_M"
    NaiveBayes = "ML_CN_5"
    NaiveBayesPredict = "ML_CN_5_P"
    NaiveBayesPredict = "ML_CN_5_M"
    DecisionTreeClassification = "ML_CN_6"
    DecisionTreeClassificationPredict = "ML_CN_6_P"
    DecisionTreeClassificationPredict = "ML_CN_6_M"
    RandomForestClassification = "ML_CN_7"
    RandomForestClassificationPredict = "ML_CN_7_P"
    RandomForestClassificationPredict = "ML_CN_7_M"
    HierarchicalClustering = "ML_CG_1"
    HierarchicalClusteringPredict = "ML_CG_1_P"
    HierarchicalClusteringPredict = "ML_CG_1_M"
    KMeansClustering = "ML_CG_2"
    KMeansClusteringPredict = "ML_CG_2_P"
    KMeansClusteringPredict = "ML_CG_2_M"
    XGBoost = "MS_XGBOOST"
    XGBoostPredict = "MS_XGBOOST_P"
    GridSearch = "ML_GRID"
    SHAPExplain = "ML_SHAP"
    IsolationForest = "ML_ISF"
    IsolationForestPredict = "ML_ISF_P"
    IsolationForestMetrics = "ML_ISF_M"

    # Auto Machine Learning
    AutoRegression = "AUTO_R_1"
    AutoRegressionPredict = "AUTO_R_1_P"
    AutoRegressionMetrics = "AUTO_R_1_M"
    AutoClassification = "AUTO_CN_1"
    AutoClassificationPredict = "AUTO_CN_1_P"
    AutoClassificationMetrics = "AUTO_CN_1_M"

    # Deep Learning
    ArtificialNeuralNetworkInit = "DL_ANN_S_I"
    ArtificialNeuralNetworkTrain = "DL_ANN_T"
    ArtificialNeuralNetworkEvaluate = "DL_ANN_E"
    ArtificialNeuralNetworkPredict = "DL_ANN_P"

    SelfOrganizingMapsInit = "DL_SOM_I"
    SelfOrganizingMapsTrain = "DL_SOM_T"
    SelfOrganizingMapsPredict = "DL_SOM_P"

    SelfOrganizingMapsInit = "DL_SOM_I"
    SelfOrganizingMapsTrain = "DL_SOM_T"
    SelfOrganizingMapsPredict = "DL_SOM_P"

    RestrictedBoltzmannMachineInit = "DL_RBM_I"
    RestrictedBoltzmannMachineTrain = "DL_RBM_T"
    RestrictedBoltzmannMachineTransform = "DL_RBM_TR"

    # Auto Deep Learnning
    AutoImageClassifierInit = "A_DL_IMC_I"
    AutoImageClassifierTrain = "A_DL_IMC_T"

    AutoImageRegressionInit = "A_DL_IMR_I"
    AutoImageRegressionTrain = "A_DL_IMR_T"

    AutoTextClassifierInit = "A_DL_TXC_I"
    AutoTextClassifierTrain = "A_DL_TXC_T"

    AutoTextRegressionInit = "A_DL_TXR_I"
    AutoTextRegressionTrain = "A_DL_TXR_T"

    AutoStructuredDataClassifierInit = "A_DL_SDC_I"
    AutoStructuredDataClassifierTrain = "A_DL_SDC_T"

    AutoStructuredDataRegressionInit = "A_DL_SDR_I"
    AutoStructuredDataRegressionTrain = "A_DL_SDR_T"

    AutoDeepLearningPredict = "A_DL_ALL_P"
    AutoDeepLearningEvaluate = "A_DL_ALL_E"



class Err(Exception):
    pass

class AutogonRequestError(Err):
    def __init__(self, status_code, error_message):
        # response status code
        self.status_code = status_code
        # the whole response header returned from API
        # self.header = header
        # error message returned from API
        self.error_message = error_message

class AutogonServerError(Err):
    def __init__(self, status_code, message):
        # response status code
        self.status_code = status_code
        # error message returned from API
        self.message = message



import requests
import base64
from io import BytesIO


class Vision:
    """Handles vision operations related to Autogon Qore."""

    endpoint = "services/"

    def __init__(self, client: any):
        """Initializes the Production class.

        Args:
            client (any): The client object for making requests.
        """
        self.client = client

    @staticmethod
    def _process_image_to_bytes(img_str: str):
        """Processes images to BytesIO objects which can be loaded by the PIL library"""
        return BytesIO(base64.b64decode(img_str.encode("utf-8")))

    @staticmethod
    def _download_image(img_url: str):
        """Downloads image from a url and processes it to bytes"""
        response = requests.get(img_url)
        return BytesIO(response.content)

    def text_detection(self, image) -> dict:
        body = {"image": image, "operation": "text_detection"}
        response = self.client.send_request(
            self.endpoint + "vision-ai/", form_data=body, method="post"
        )
        return response

    def document_text_detection(self, image) -> dict:
        body = {"image": image, "operation": "document_text_detection"}
        response = self.client.send_request(
            self.endpoint + "vision-ai/", form_data=body, method="post"
        )
        return response

    def label_detection(self, image) -> dict:
        body = {"image": image, "operation": "label_detection"}
        response = self.client.send_request(
            self.endpoint + "vision-ai/", form_data=body, method="post"
        )
        return response

    def landmark_detection(self, image) -> dict:
        body = {"image": image, "operation": "landmark_detection"}
        response = self.client.send_request(
            self.endpoint + "vision-ai/", form_data=body, method="post"
        )
        return response

    def logo_detection(self, image) -> dict:
        body = {"image": image, "operation": "logo_detection"}
        response = self.client.send_request(
            self.endpoint + "vision-ai/", form_data=body, method="post"
        )
        return response

    def web_detection(self, image) -> dict:
        body = {"image": image, "operation": "web_detection"}
        response = self.client.send_request(
            self.endpoint + "vision-ai/", form_data=body, method="post"
        )
        return response

    def object_detection(self, image) -> dict:
        body = {"image": image, "operation": "object_detection"}
        response = self.client.send_request(
            self.endpoint + "vision-ai/", form_data=body, method="post"
        )
        return response

    def image_generation(self, prompt, output_size) -> dict:
        body = {"prompt": prompt, "output_size": output_size}
        response = self.client.send_request(
            self.endpoint + "image-generation/", form_data=body, method="post"
        )
        response["image"] = self._process_image_to_bytes(response["image"])
        return response

    def stable_diffusion(self, text) -> dict:
        body = {"text": text}
        response = self.client.send_request(
            self.endpoint + "stable-diffusion/", json_data=body, method="post"
        )
        return response

    def image_captioning(self, image, image_url) -> dict:
        body = {"image": image, "image_url": image_url}
        response = self.client.send_request(
            self.endpoint + "image-captioning/", form_data=body, method="post"
        )
        return response

    def document_qa(self, question, document, document_url) -> dict:
        body = {
            "question": question,
            "document": document,
            "document_url": document_url,
        }
        response = self.client.send_request(
            self.endpoint + "document-qa/", form_data=body, method="post"
        )
        return response

    def visual_qa(self, text, image, image_url) -> dict:
        body = {
            "text": text,
            "image": image,
            "image_url": image_url,
        }
        response = self.client.send_request(
            self.endpoint + "viit-vq/", form_data=body, method="post"
        )
        return response


class NaturalLanguage:
    """Handles vision operations related to Autogon Qore."""

    endpoint = "services/"

    def __init__(self, client: any):
        """Initializes the Production class.

        Args:
            client (any): The client object for making requests.
        """
        self.client = client

    def sentiment_analysis(self, text) -> dict:
        body = {"text": text}
        response = self.client.send_request(
            self.endpoint + "sentiment-analysis/", json_data=body, method="post"
        )
        return response

    def text_to_speech(self, text, language_code) -> dict:
        body = {"text": text, "language_code": language_code}
        response = self.client.send_request(
            self.endpoint + "text-to-speech/", json_data=body, method="post"
        )
        return response

    def speech_to_text(self, audio, language_code) -> dict:
        body = {"audio": audio, "language_code": language_code}
        response = self.client.send_request(
            self.endpoint + "speech-to-text/", form_data=body, method="post"
        )
        return response

    def text_summary(self, text, max_length, min_length) -> dict:
        body = {"text": text, "max_length": max_length, "min_length": min_length}
        response = self.client.send_request(
            self.endpoint + "text-summary/", json_data=body, method="post"
        )
        return response

    def ask_your_data(self, data, prompt) -> dict:
        body = {"data": data, "prompt": prompt}
        response = self.client.send_request(
            self.endpoint + "ask-your-data/", json_data=body, method="post"
        )
        return response

    def chatgpt(self, message) -> dict:
        body = {"message": message}
        response = self.client.send_request(
            self.endpoint + "chat/", json_data=body, method="post"
        )
        return response

    def text_classification(self, text) -> dict:
        body = {"text": text}
        response = self.client.send_request(
            self.endpoint + "text-classification/", json_data=body, method="post"
        )
        return response


class Voice:
    """Handles operations related to Autogon Qore."""

    endpoint = "services/"

    def __init__(self, client: any):
        """Initializes the Production class.

        Args:
            client (any): The client object for making requests.
        """
        self.client = client

    def create_voice(self, voice_name, voice_description, audio) -> dict:
        body = {
            "voice_name": voice_name,
            "voice_description": voice_description,
            "audio": audio,
        }
        response = self.client.send_request(
            self.endpoint + "voice-cloning/voices/", form_data=body, method="post"
        )
        return response

    def get_voices(self) -> dict:
        response = self.client.send_request(
            self.endpoint + "voice-cloning/voices/list/", method="get"
        )
        return response

    def text_to_speech(self, voice_id, text) -> dict:
        body = {
            "voice_id": voice_id,
            "text": text,
        }
        response = self.client.send_request(
            self.endpoint + "voice-cloning/tts/", form_data=body, method="post"
        )
        return response



# Import necessary libraries
from autogonai.development.base import BaseBlock, gen_params


# Define class DataInput
class DataInput(BaseBlock):
    def set_params(
        self,
        # file_type: str = None,
        # dburl: str = None,
        # x_boundaries: str = None,
        # y_boundaries: str = None,
        # dbservertype: str = None,
        # dbpassword: str = None,
        # dbname: str = None,
        # query: str = None,
        **kwargs
    ):
        """
        Sets parameters for DataInput class.

        Args:
            file_type (str, optional): Type of the input file. Defaults to None.
            dburl (str, optional): URL of the database. Defaults to None.
            x_boundaries (str, optional): X boundaries for data. Defaults to None.
            y_boundaries (str, optional): Y boundaries for data. Defaults to None.
            dbservertype (str, optional): Type of the database server. Defaults to None.
            dbpassword (str, optional): Password for the database. Defaults to None.
            dbname (str, optional): Name of the database. Defaults to None.
            query (str, optional): Query for database retrieval. Defaults to None.
            Other args can be found at: https://docs.autogon.ai/autogon-engine-studio/data-processing/data-input-dp_1

        Returns:
            dict: Parameters dictionary.
        """
        # self.params = gen_params(locals())
        self.params = kwargs
        return self.params


# Define class HandleMissingData
class HandleMissingData(BaseBlock):
    def set_params(
        self,
        strategy_value: str = None,
        boundaries: str | None = None,
    ):
        """
        Sets parameters for HandleMissingData class.

        Args:
            strategy_value (str, optional): Strategy for handling missing data. Defaults to None.
            boundaries (str, optional): Dataset boundaries for data. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class DataEncoding
class DataEncoding(BaseBlock):
    def set_params(
        self,
        dataset={"encode": False},
        xvalue={"encode": False},
        yvalue={"encode": False},
        save_name=None,
        load_name=None,
    ):
        """
        Sets parameters for DataEncoding class.

        Args:
            dataset (dict): Encoding Parameters for dataset variable
            xvalue (dict): Encoding Parameters for x variable
            yvalue (dict): Encoding Parameters for y variable
            save_name (optional): Save name for encoded data. Defaults to None.
            load_name (optional): Load name for encoded data. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class DataSplitting
class DataSplitting(BaseBlock):
    def set_params(self, test_size: float = None, random_state: int | None = None):
        """
        Sets parameters for DataSplitting class.

        Args:
            test_size (float, optional): Size of the test data. Defaults to None.
            random_state (int, optional): Random state for data splitting. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class FeatureScaling
class FeatureScaling(BaseBlock):
    def set_params(
        self,
        dataset: bool,
        xtrain: bool = None,
        xtest: bool = None,
        x: bool = None,
        ytrain: bool = None,
        ytest: bool = None,
        y: bool = None,
        boundariestoscale: str = None,
        save_name=None,
        load_name=None,
    ):
        """
        Sets parameters for FeatureScaling class.

        Args:
            xtrain (bool, optional): Flag for scaling X training data. Defaults to None.
            xtest (bool, optional): Flag for scaling X test data. Defaults to None.
            x (bool, optional): Flag for scaling X data. Defaults to None.
            ytrain (bool, optional): Flag for scaling Y training data. Defaults to None.
            ytest (bool, optional): Flag for scaling Y test data. Defaults to None.
            y (bool, optional): Flag for scaling Y data. Defaults to None.
            boundariestoscale (str, optional): Boundaries for scaling. Defaults to None.
            save_name (optional): Save name for scaled data. Defaults to None.
            load_name (optional): Load name for scaled data. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class DropColumns
class DropColumns(BaseBlock):
    def set_params(
        self,
        d_columns: list | None = None,
        x_columns: list | None = None,
        y_columns: list | None = None,
    ):
        """
        Sets parameters for DropColumns class.

        Args:
            d_columns (list, optional): Columns to drop from dataset. Defaults to None.
            x_columns (list, optional): Columns to drop from X data. Defaults to None.
            y_columns (list, optional): Columns to drop from Y data. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class TimeStepData
class TimeStepData(BaseBlock):
    def set_params(
        self,
        index: int,
        lookback: int,
        lookforward: int = 1,
    ):
        """
        Sets parameters for TimeStepData class.

        Args:
            index (int): Column index in the dataset
            lookback (int): The lookback value. Defaults to None.
            lookback (int, optional): The lookforward value. Defaults to 1.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class FeatureSampling
class FeatureSampling(BaseBlock):
    def set_params(
        self,
        dataset_url: str = None,
        x_boundaries=None,
        y_boundaries=None,
    ):
        """
        Sets parameters for FeatureSampling class.

        Args:
            dataset_url (str, optional): URL of the dataset. Defaults to None.
            x_boundaries (optional): X boundaries for data sampling. Defaults to None.
            y_boundaries (optional): Y boundaries for data sampling. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class ReshapeArray
class ReshapeArray(BaseBlock):
    def set_params(
        self,
        data=None,
        dimensions=None,
    ):
        """
        Sets parameters for ReshapeArray class.

        Args:
            data (optional): Data to reshape. Defaults to None.
            dimensions (optional): Dimensions for reshaping. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class ScalarToNdarray
class ScalarToNdarray(BaseBlock):
    def set_params(
        self,
        scalar=None,
        dimensions: int = None,
    ):
        """
        Sets parameters for ScalarToNdarray class.

        Args:
            scalar (optional): Scalar value to convert to ndarray. Defaults to None.
            dimensions (int, optional): Dimensions of the ndarray. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class ImageToNdarray
class ImageToNdarray(BaseBlock):
    def set_params(self, image_url: str = None, targe_size=None, rescale: bool = True):
        """
        Sets parameters for ImageToNdarray class.

        Args:
            image_url (str, optional): URL of the image. Defaults to None.
            targe_size (optional): Target size for the image. Defaults to None.
            rescale (bool, optional): Flag for image rescaling. Defaults to True.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class ColumnsAstype
class ColumnsAstype(BaseBlock):
    def set_params(
        self,
        columns=None,
        astype: str = None,
    ):
        """
        Sets parameters for ColumnsAstype class.

        Args:
            columns (optional): Columns to apply astype conversion. Defaults to None.
            astype (str, optional): Data type for the conversion. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoDP
class AutoDP(BaseBlock):
    def set_params(
        self,
        x_slice=None,
        y_slice=None,
        strategy_value=None,
        test_size_value=None,
        le_thresh=None,
        ohe_thresh=None,
        save_name=None,
        clean=None,
        dataset_type=None,
        load_name=None,
    ):
        """
        Sets parameters for AutoDP class.

        Args:
            x_slice (optional): X data slice. Defaults to None.
            y_slice (optional): Y data slice. Defaults to None.
            strategy_value (optional): Strategy for data processing. Defaults to None.
            test_size_value (optional): Size of the test data. Defaults to None.
            le_thresh (optional): Threshold for Label Encoding. Defaults to None.
            ohe_thresh (optional): Threshold for One-Hot Encoding. Defaults to None.
            save_name (optional): Save name for processed data. Defaults to None.
            clean (optional): Data cleaning flag. Defaults to None.
            dataset_type (optional): Type of the dataset. Defaults to None.
            load_name (optional): Load name for processed data. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class TextVectorizer
class TextVectorizer(BaseBlock):
    def set_params(
        self,
        vectorizer=None,
        boundariestoscale=None,
        dataset=None,
        xtrain=None,
        xtest=None,
        x=None,
        ytrain=None,
        ytest=None,
        y=None,
        save_name=None,
        load_name=None,
    ):
        """
        Sets parameters for TextVectorizer class.

        Args:
            vectorizer (optional): Text vectorizer. Defaults to None.
            boundariestoscale (optional): Boundaries for scaling. Defaults to None.
            dataset (optional): Dataset. Defaults to None.
            xtrain (optional): X training data. Defaults to None.
            xtest (optional): X test data. Defaults to None.
            x (optional): X data. Defaults to None.
            ytrain (optional): Y training data. Defaults to None.
            ytest (optional): Y test data. Defaults to None.
            y (optional): Y data. Defaults to None.
            save_name (optional): Save name for processed data. Defaults to None.
            load_name (optional): Load name for processed data. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class ParseDatetime
class ParseDatetime(BaseBlock):
    def set_params(self, index=None, drop=None):
        """
        Sets parameters for ParseDatetime class.

        Args:
            index (optional): Index for date parsing. Defaults to None.
            drop (optional): Drop flag for date parsing. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class ReorderColumn
class ReorderColumn(BaseBlock):
    def set_params(
        self,
        column=None,
        posititon=None,
        dataset=None,
        xtrain=None,
        xtest=None,
        x=None,
        ytrain=None,
        ytest=None,
        y=None,
    ):
        """
        Sets parameters for ReorderColumn class.

        Args:
            column (optional): Column to reorder. Defaults to None.
            posititon (optional): New position for the column. Defaults to None.
            dataset (optional): Dataset. Defaults to None.
            xtrain (optional): X training data. Defaults to None.
            xtest (optional): X test data. Defaults to None.
            x (optional): X data. Defaults to None.
            ytrain (optional): Y training data. Defaults to None.
            ytest (optional): Y test data. Defaults to None.
            y (optional): Y data. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class ColumnAstype
class ColumnAstype(BaseBlock):
    def set_params(self, astype=None, columns=None):
        """
        Sets parameters for ColumnAstype class.

        Args:
            astype (optional): Data type for columns. Defaults to None.
            columns (optional): Columns to convert. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class ShowDuplicates
class ShowDuplicates(BaseBlock):
    def set_params(self, columns=None):
        """
        Sets parameters for ShowDuplicates class.

        Args:
            columns (optional): Columns to check for duplicates. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class DropDuplicates
class DropDuplicates(BaseBlock):
    def set_params(self, columns=None):
        """
        Sets parameters for DropDuplicates class.

        Args:
            columns (optional): Columns to drop duplicates. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class DatasetInfo
class DatasetInfo(BaseBlock):
    def set_params(self):
        """
        Sets parameters for DatasetInfo class.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class DatasetCorrelations
class DatasetCorrelations(BaseBlock):
    def set_params(self):
        """
        Sets parameters for DatasetCorrelations class.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class DatasetDescription
class DatasetDescription(BaseBlock):
    def set_params(self):
        """
        Sets parameters for DatasetDescription class.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class DatasetUniques
class DatasetUniques(BaseBlock):
    def set_params(self):
        """
        Sets parameters for DatasetUniques class.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class StatsCounts
class StatsCounts(BaseBlock):
    def set_params(self, index=None):
        """
        Sets parameters for StatsCounts class.

        Args:
            index (optional): Index for stats counting. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class PrincipalComponentAnalysis
class PrincipalComponentAnalysis(BaseBlock):
    def set_params(
        self,
        n_components=None,
        dataset=None,
        xtrain=None,
        xtest=None,
        x=None,
        ytrain=None,
        ytest=None,
        y=None,
        save_name=None,
        load_name=None,
    ):
        """
        Sets parameters for PrincipalComponentAnalysis class.

        Args:
            n_components (optional): Number of components for PCA. Defaults to None.
            dataset (optional): Dataset. Defaults to None.
            xtrain (optional): X training data. Defaults to None.
            xtest (optional): X test data. Defaults to None.
            x (optional): X data. Defaults to None.
            ytrain (optional): Y training data. Defaults to None.
            ytest (optional): Y test data. Defaults to None.
            y (optional): Y data. Defaults to None.
            save_name (optional): Save name for PCA data. Defaults to None.
            load_name (optional): Load name for PCA data. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class Resampler
class Resampler(BaseBlock):
    def set_params(
        self,
        xy=None,
        xy_train=None,
        xy_test=None,
        resampler=None,
        save_name=None,
        load_name=None,
    ):
        """
        Sets parameters for Resampler class.

        Args:
            xy (optional): Data for resampling. Defaults to None.
            xy_train (optional): Training data for resampling. Defaults to None.
            xy_test (optional): Test data for resampling. Defaults to None.
            resampler (optional): Resampling strategy. Defaults to None.
            save_name (optional): Save name for resampled data. Defaults to None.
            load_name (optional): Load name for resampled data. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class ScatterPlot
class ScatterPlot(BaseBlock):
    def set_params(self, xvalue=None, yvalue=None, is_grid=None):
        """
        Sets parameters for ScatterPlot class.

        Args:
            xvalue (optional): X values for the plot. Defaults to None.
            yvalue (optional): Y values for the plot. Defaults to None.
            is_grid (optional): Grid flag for the plot. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class OrdinaryPlot
class OrdinaryPlot(BaseBlock):
    def set_params(self, xvalue=None, yvalue=None, is_grid=None):
        """
        Sets parameters for OrdinaryPlot class.

        Args:
            xvalue (optional): X values for the plot. Defaults to None.
            yvalue (optional): Y values for the plot. Defaults to None.
            is_grid (optional): Grid flag for the plot. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class CompareScatterPlots
class CompareScatterPlots(BaseBlock):
    def set_params(
        self,
        avalue=None,
        bvalue=None,
        xvalue=None,
        yvalue=None,
        is_grid=None,
    ):
        """
        Sets parameters for CompareScatterPlots class.

        Args:
            avalue (optional): Data A for the plot. Defaults to None.
            bvalue (optional): Data B for the plot. Defaults to None.
            xvalue (optional): X values for the plot. Defaults to None.
            yvalue (optional): Y values for the plot. Defaults to None.
            is_grid (optional): Grid flag for the plot. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class PiePlot
class PiePlot(BaseBlock):
    def set_params(self, dataset=None):
        """
        Sets parameters for PiePlot class.

        Args:
            dataset (optional): Dataset for the pie plot. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class HeatmapPlot
class HeatmapPlot(BaseBlock):
    def set_params(self, dataset=None):
        """
        Sets parameters for HeatmapPlot class.

        Args:
            dataset (optional): Dataset for the heatmap plot. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params



# Import necessary libraries
import json
import requests
from .errors import AutogonRequestError, AutogonServerError


class API:
    def __init__(
        self,
        api_key,
        base_url=None,
        timeout=None,
        show_header=False,
        show_request=True,
        proxies=None,
    ):
        """
        Initialize the API class.

        Args:
            api_key (str): API key for authentication. Defaults to None.
            base_url (str, optional): Base URL for API requests. Defaults to None.
            timeout (int, optional): Timeout for API requests in seconds. Defaults to None.
            show_header (bool, optional): Flag to show request header. Defaults to False.
            show_request (bool, optional): Flag to show request details. Defaults to True.
            proxies (dict, optional): Proxy settings for API requests. Defaults to None.
        """
        # Initialize class attributes
        if not base_url:
            self.url = "https://api.autogon.ai/api/v1/"
        else:
            self.url = base_url
        self.api_key = api_key
        self.header = {"X-AUG-KEY": api_key}

    # Define a method to send HTTP requests
    def send_request(self, endpoint, json_data=None, form_data=None, method="post"):
        """
        Send an HTTP request to the specified endpoint.

        Args:
            endpoint (str): API endpoint to send the request to.
            payload (dict, optional): Data to be sent with the request. Defaults to {}.
            method (str, optional): HTTP method for the request (post, get, patch, put, delete). Defaults to "post".

        Returns:
            dict: JSON response from the API.
        """
        url = self.url + endpoint

        # Send an HTTP request based on the specified method
        if method.lower() == "post":
            response = requests.post(
                url, headers=self.header, json=json_data, data=form_data
            )
        elif method.lower() == "get":
            response = requests.get(url, headers=self.header)
        elif method.lower() == "patch":
            response = requests.patch(
                url, headers=self.header, json=json_data, data=form_data
            )
        elif method.lower() == "put":
            response = requests.put(
                url, headers=self.header, json=json_data, data=form_data
            )
        elif method.lower() == "delete":
            response = requests.delete(url, headers=self.header)

        # Handle exceptions that may occur during the request

        try:
            data = response.json()
        except ValueError:
            data = response.text

        if type(data) == str:
            # print(data)
            raise AutogonServerError(response.status_code, data)

        if data.get("status") == "false":
            raise AutogonRequestError(response.status_code, data["message"])

        # Check the response data for errors and raise exceptions if necessary
        if type(data) == str:
            raise AutogonServerError(response.status_code, data)

        if data.get("status") in ["false", False]:
            raise AutogonRequestError(response.status_code, data["message"])

        return data



from autogonai.development.base import BaseBlock, gen_params


class PredictBlock(BaseBlock):
    def set_params(self, model_name: str, test_data: str):
        """SimpleLinearRegressionPredict Parameters"""
        self.params = gen_params(locals())
        return self.params


class MetricsBlock(BaseBlock):
    def set_params(self, model_name: str, metric: str):
        """SimpleLinearRegressionPredict Parameters"""
        self.params = gen_params(locals())
        return self.params


class SimpleLinearRegression(BaseBlock):
    def set_params(self, model_name: str):
        """SimpleLinearRegression Parameters"""
        self.params = gen_params(locals())
        return self.params


class SimpleLinearRegressionPredict(PredictBlock):
    pass


class SimpleLinearRegressionMetrics(MetricsBlock):
    pass


class MultipleLinearRegression(BaseBlock):
    def set_params(self, model_name: str):
        """SimpleLinearRegression Parameters"""
        self.params = gen_params(locals())
        return self.params


class MultipleLinearRegressionPredict(PredictBlock):
    pass


class MultipleLinearRegressionMetrics(MetricsBlock):
    pass


class PolynomialLinearRegression(BaseBlock):
    def set_params(self, model_name: str, degree: int = 2):
        """PolynomialLinearRegression Parameters"""
        self.params = gen_params(locals())
        return self.params


class PolynomialLinearRegressionPredict(PredictBlock):
    pass


class PolynomialLinearRegressionMetrics(MetricsBlock):
    pass


class SupportVectorRegression(BaseBlock):
    def set_params(self, model_name: str, kernel: str = "rbf"):
        """SupportVectorRegression Parameters"""
        self.params = gen_params(locals())
        return self.params


class SupportVectorRegressionPredict(PredictBlock):
    pass


class SupportVectorRegressionMetrics(MetricsBlock):
    pass


class DecisionTreeRegression(BaseBlock):
    def set_params(self, model_name: str, random_state: int = 0):
        """DecisionTreeRegression Parameters"""
        self.params = gen_params(locals())
        return self.params


class DecisionTreeRegressionPredict(PredictBlock):
    pass


class DecisionTreeRegressionMetrics(MetricsBlock):
    pass


class RandomForestRegression(BaseBlock):
    def set_params(
        self, model_name: str, n_estimators: int = 100, random_state: int = 0
    ):
        """RandomForestRegression Parameters"""
        self.params = gen_params(locals())
        return self.params


class RandomForestRegressionPredict(PredictBlock):
    pass


class RandomForestRegressionMetrics(MetricsBlock):
    pass


class LogisticRegression(BaseBlock):
    def set_params(self, model_name: str, random_state: int = 0):
        """LogisticRegression Parameters"""
        self.params = gen_params(locals())
        return self.params


class LogisticRegressionPredict(PredictBlock):
    pass


class LogisticRegressionMetrics(MetricsBlock):
    pass


class KNearestNeighbors(BaseBlock):
    def set_params(
        self,
        model_name: str,
        n_neighbors: int = 5,
        distance: str = "minkowski",
        p: int = 2,
        random_state: int = 0,
    ):
        """KNearestNeighbors Parameters"""
        self.params = gen_params(locals())
        return self.params


class KNearestNeighborsPredict(PredictBlock):
    pass


class KNearestNeighborsMetrics(MetricsBlock):
    pass


class SupportVectorMachine(BaseBlock):
    def set_params(self, model_name: str):
        """SupportVectorMachine Parameters"""
        self.params = gen_params(locals())
        return self.params


class SupportVectorMachinePredict(PredictBlock):
    pass


class SupportVectorMachineMetrics(MetricsBlock):
    pass


class KernelSupportVectorMachine(BaseBlock):
    def set_params(self, model_name: str, kernel: str = "rbf", random_state: int = 0):
        """KernelSupportVectorMachine Parameters"""
        self.params = gen_params(locals())
        return self.params


class KernelSupportVectorMachinePredict(PredictBlock):
    pass


class KernelSupportVectorMachineMetrics(MetricsBlock):
    pass


class NaiveBayes(BaseBlock):
    def set_params(self, model_name: str):
        """NaiveBayes Parameters"""
        self.params = gen_params(locals())
        return self.params


class NaiveBayesPredict(PredictBlock):
    pass


class NaiveBayesMetrics(MetricsBlock):
    pass


class DecisionTreeClassification(BaseBlock):
    def set_params(
        self, model_name: str, criterion: str = "gini", random_state: int = 0
    ):
        """DecisionTreeClassification Parameters"""
        self.params = gen_params(locals())
        return self.params


class DecisionTreeClassificationPredict(PredictBlock):
    pass


class DecisionTreeClassificationMetrics(MetricsBlock):
    pass


class RandomForestClassification(BaseBlock):
    def set_params(self, model_name: str):
        """RandomForestClassification Parameters"""
        self.params = gen_params(locals())
        return self.params


class RandomForestClassificationPredict(PredictBlock):
    pass


class RandomForestClassificationMetrics(MetricsBlock):
    pass


class HierarchicalClustering(BaseBlock):
    def set_params(
        self,
        model_name: str,
        n_clusters: int = 5,
        affinity: str = "euclidean",
        linkage: str = "ward",
    ):
        """HierarchichalClustering Parameters"""
        self.params = gen_params(locals())
        return self.params


class HierarchicalClusteringPredict(PredictBlock):
    pass


class HierarchicalClusteringMetrics(MetricsBlock):
    pass


class KMeansClustering(BaseBlock):
    def set_params(
        self,
        model_name: str,
        n_clusters: int = 5,
        init: str = "k-means++",
        random_state: int = 0,
    ):
        """KMeansClustering Parameters"""
        self.params = gen_params(locals())
        return self.params


class KMeansClusteringPredict(PredictBlock):
    pass


class KMeansClusteringMetrics(MetricsBlock):
    pass


class XGBoost(BaseBlock):
    def set_params(
        self,
        model_name: str,
    ):
        """XGBoost Parameters"""
        self.params = gen_params(locals())
        return self.params


class XGBoostPredict(PredictBlock):
    pass


class XGBoostMetrics(MetricsBlock):
    pass


class GridSearch(BaseBlock):
    def set_params(
        self,
        model_name: str,
        param_grid: list,
    ):
        """GridSearch Parameters"""
        self.params = gen_params(locals())
        return self.params


class SHAPExplain(BaseBlock):
    def set_params(
        self,
        model_name: str,
    ):
        """SHAPExplain Parameters"""
        self.params = gen_params(locals())
        return self.params


class IsolationForest(BaseBlock):
    def set_params(
        self,
        model_name: str,
        random_state: int,
    ):
        """IsolationForest Parameters"""
        self.params = gen_params(locals())
        return self.params


class IsolationForestPredict(PredictBlock):
    pass


class IsolationForestMetrics(MetricsBlock):
    pass


class DBScanClustering(BaseBlock):
    def set_params(self, eps, min_samples, metric):
        """DBScanClustering Parameters"""
        self.params = gen_params(locals())
        return self.params


class DBScanClusteringPredict(PredictBlock):
    pass



import json


def gen_params(arg_pairs):
    # Create an empty dictionary to store the arguments
    arguments_dict = {}

    # Get all local variables (including function arguments) as a dictionary
    local_variables = arg_pairs

    # Iterate through the local variables and store the arguments in the dictionary
    for key, value in local_variables.items():
        if key != "self":  # Exclude 'self' if it's a method within a class
            arguments_dict[key] = value

    return arguments_dict


class BaseBlock(int):
    endpoint = "engine/start"

    def __new__(cls, data):
        i = int.__new__(cls, data["block_id"])
        i.client = data["client"]
        i.project_id = data["project_id"]
        i.block_id = data["block_id"]
        i.parent_id = data["parent_id"]
        i.function_code = data["function_code"]
        i.params = {}
        return i

    def set_params(self, **kwargs):
        self.params = kwargs
        return self.params

    def run(self):
        body = {
            "project_id": self.project_id,
            "parent_id": self.parent_id,
            "block_id": self.block_id,
            "function_code": self.function_code,
            "args": self.params,
        }
        response = self.client.send_request(self.endpoint, body)
        return response



from .qore import *


class Projects:
    """Handles operations related to projects."""

    endpoint = "engine/project/"

    def __init__(self, client: any):
        """Initializes the Projects class.

        Args:
            client (any): The client object for making requests.
        """
        self.client = client

    def get(self, app_id) -> dict:
        """Fetches an existing Project by its UUID.

        Args:
            app_id (str): UUID of the project.

        Returns:
            dict: Project details.
        """
        response = self.client.send_request(self.endpoint + app_id, method="get")
        return response

    def create(self, name: str, description: str) -> dict:
        """Creates a new project.

        Args:
            name (str): Name of the project.
            description (str): Short description for the project.

        Returns:
            dict: Project details.
        """
        body = {"project_name": name, "project_description": description}
        response = self.client.send_request(self.endpoint, body)
        return response

    def delete(self, app_id) -> str:
        """Deletes a project by its UUID.

        Args:
            app_id (str): UUID of the project.

        Returns:
            str: Confirmation of the deletion.
        """
        response = self.client.send_request(self.endpoint + app_id, method="delete")
        return response


# class Project(int):
#     """Represents a project with additional metadata."""

#     data = {}
#     id = 0
#     app_id = ""
#     name = ""
#     description = ""
#     compiled_models = {}

#     def __new__(cls, data: dict):
#         """Creates a new Project instance.

#         Args:
#             data (dict): Project data.

#         Returns:
#             Project: Project instance.
#         """
#         i = int.__new__(cls, data["id"])
#         i.data = data
#         i.id = data["id"]
#         i.app_id = data["app_id"]
#         i.name = data["project_name"]
#         i.description = data["project_description"]
#         i.compiled_models = data["project_compiled_models"]
#         return i


class StateManagements:
    """Handles operations related to state management."""

    endpoint = "engine/statemanagement/"

    def __init__(self, client: any):
        """Initializes the StateManagements class.

        Args:
            client (any): The client object for making requests.
        """
        self.client = client

    def get(self, id) -> dict:
        """Fetches state management details by ID.

        Args:
            id (int): ID of the state management.

        Returns:
            dict: State management details.
        """
        response = self.client.send_request(self.endpoint + str(id), method="get")
        return response

    def list(self) -> dict:
        """Lists all available state managements.

        Returns:
            dict: List of state managements.
        """
        response = self.client.send_request(self.endpoint, method="get")
        return response

    def delete(self, id) -> dict:
        """Deletes a state management by ID.

        Args:
            id (int): ID of the state management.

        Returns:
            dict: Confirmation of the deletion.
        """
        response = self.client.send_request(self.endpoint + str(id), method="delete")
        return response


class Datasets:
    """Handles operations related to datasets."""

    endpoint = "engine/dataset/"

    def __init__(self, client: any):
        """Initializes the Datasets class.

        Args:
            client (any): The client object for making requests.
        """
        self.client = client

    def get(self, id) -> dict:
        """Fetches dataset details by ID.

        Args:
            id (int): ID of the dataset.

        Returns:
            dict: Dataset details.
        """
        response = self.client.send_request(self.endpoint + str(id), method="get")
        return response

    def list(self) -> dict:
        """Lists all available datasets.

        Returns:
            dict: List of datasets.
        """
        response = self.client.send_request(self.endpoint, method="get")
        return response

    def create(self, name: str, description: str, dataset_url: str) -> dict:
        """Creates a new dataset.

        Args:
            name (str): Name of the dataset.
            description (str): Description of the dataset.
            dataset_url (str): URL of the dataset.

        Returns:
            dict: Dataset details.
        """
        body = {"name": name, "description": description, "dataset_url": dataset_url}
        response = self.client.send_request(
            self.endpoint, json_data=body, method="post"
        )
        return response

    def update(self, id, name: str, description: str) -> dict:
        """Updates an existing dataset.

        Args:
            id (int): ID of the dataset to update.
            name (str): New name for the dataset.
            description (str): New description for the dataset.

        Returns:
            dict: Dataset details after the update.
        """
        body = {"dataset_name": name, "description": description}
        response = self.client.send_request(
            self.endpoint + str(id), json_data=body, method="post"
        )
        return response

    def delete(self, id) -> dict:
        """Deletes a dataset by ID.

        Args:
            id (int): ID of the dataset to delete.

        Returns:
            dict: Confirmation of the deletion.
        """
        response = self.client.send_request(self.endpoint + str(id), method="delete")
        return response


class Production:
    """Handles operations related to production pipelines."""

    endpoint = "models/"

    def __init__(self, client: any):
        """Initializes the Production class.

        Args:
            client (any): The client object for making requests.
        """
        self.client = client

    def generate_dataset(self, data: dict) -> dict:
        """Generates a dataset based on the provided data.

        Args:
            data (dict): Data to generate the dataset.

        Returns:
            dict: Generated dataset details.
        """
        body = {"data": data}
        response = self.client.send_request(
            self.endpoint + "generate/", json_data=body, method="post"
        )
        return response

    def run_pipeline(self, flow_id, data: dict) -> dict:
        """Runs a production pipeline with the given flow ID and test data.

        Args:
            flow_id: ID of the flow to run.
            data (dict): Test data for the pipeline.

        Returns:
            dict: Results of the production pipeline run.
        """
        body = {"flow_id": flow_id, "data": data}
        response = self.client.send_request(
            self.endpoint + "production/", json_data=body, method="post"
        )
        return response


class Qore:
    def __init__(self, client: any):
        """Initializes the Production class.

        Args:
            client (any): The client object for making requests.
        """
        self.client = client

        self.VisionAI = Vision(client)
        self.NaturalLanguageAI = NaturalLanguage(client)
        self.Voice = Voice(client)



# Import necessary libraries
from autogonai.development.base import BaseBlock, gen_params


# Define class ArtificialNeuralNetworkInit
class ArtificialNeuralNetworkInit(BaseBlock):
    def set_params(self, layer_list: list = None):
        """
        Sets parameters for ArtificialNeuralNetworkInit class.

        Args:
            layer_list (list, optional): List of layers for the neural network. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class ArtificialNeuralNetworkTrain
class ArtificialNeuralNetworkTrain(BaseBlock):
    def set_params(
        self, model_name: str = None, add_dim: bool = None, hyp_params: dict = None
    ):
        """
        Sets parameters for ArtificialNeuralNetworkTrain class.

        Args:
            model_name (str, optional): Name of the neural network model. Defaults to None.
            add_dim (bool, optional): Flag for adding dimensions. Defaults to None.
            hyp_params (dict, optional): Hyperparameters for training. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class ArtificialNeuralNetworkEvaluate
class ArtificialNeuralNetworkEvaluate(BaseBlock):
    def set_params(self, hyp_params: dict = None):
        """
        Sets parameters for ArtificialNeuralNetworkEvaluate class.

        Args:
            hyp_params (dict, optional): Hyperparameters for evaluation. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class ArtificialNeuralNetworkPredict
class ArtificialNeuralNetworkPredict(BaseBlock):
    def set_params(self, test_data: str = None, hyp_params: dict = None):
        """
        Sets parameters for ArtificialNeuralNetworkPredict class.

        Args:
            test_data (str, optional): Test data for prediction. Defaults to None.
            hyp_params (dict, optional): Hyperparameters for prediction. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class SelfOrganizingMapsInit
class SelfOrganizingMapsInit(BaseBlock):
    def set_params(self, hyp_params: dict = None):
        """
        Sets parameters for SelfOrganizingMapsInit class.

        Args:
            hyp_params (dict, optional): Hyperparameters for SOM initialization. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class SelfOrganizingMapsTrain
class SelfOrganizingMapsTrain(BaseBlock):
    def set_params(self, model_name: str = None, hyp_params: dict = None):
        """
        Sets parameters for SelfOrganizingMapsTrain class.

        Args:
            model_name (str, optional): Name of the SOM model. Defaults to None.
            hyp_params (dict, optional): Hyperparameters for SOM training. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class SelfOrganizingMapsPredict
class SelfOrganizingMapsPredict(BaseBlock):
    def set_params(self, test_data: str = None, row: int = None):
        """
        Sets parameters for SelfOrganizingMapsPredict class.

        Args:
            test_data (str, optional): Test data for prediction. Defaults to None.
            row (int, optional): Row for SOM prediction. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class RestrictedBoltzmannMachineInit
class RestrictedBoltzmannMachineInit(BaseBlock):
    def set_params(self, hyp_params: dict = None):
        """
        Sets parameters for RestrictedBoltzmannMachineInit class.

        Args:
            hyp_params (dict, optional): Hyperparameters for RBM initialization. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class RestrictedBoltzmannMachineTrain
class RestrictedBoltzmannMachineTrain(BaseBlock):
    def set_params(self, model_name: str = None, hyp_params: dict = None):
        """
        Sets parameters for RestrictedBoltzmannMachineTrain class.

        Args:
            model_name (str, optional): Name of the RBM model. Defaults to None.
            hyp_params (dict, optional): Hyperparameters for RBM training. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class RestrictedBoltzmannMachineTransform
class RestrictedBoltzmannMachineTransform(BaseBlock):
    def set_params(self, test_data: str = None):
        """
        Sets parameters for RestrictedBoltzmannMachineTransform class.

        Args:
            test_data (str, optional): Test data for transformation. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params



class Blocks:
    import autogonai.development.data_processing as dp
    import autogonai.development.machine_learning as ml
    import autogonai.development.auto_machine_learning as aml
    import autogonai.development.deep_learning as dl
    import autogonai.development.auto_deep_learning as adl

    from autogonai.constants import function_codes as fc
    from autogonai.development.base import BaseBlock

    endpoint = "engine/start"

    def __init__(self, client):
        self.client = client

    def get(self):
        pass

    def new(  # Slower structure to implemnt
        self,
        function_code: str,
        project_id: int,
        id: int,
        parent: int = 0,
    ) -> BaseBlock:
        data = {
            "client": self.client,
            "project_id": project_id,
            "parent_id": parent,
            "block_id": id,
            "function_code": function_code,
        }

        functions = vars(self.fc)

        # Main functions finder
        if function_code in functions.values():
            function_name = [k for k, v in functions.items() if v == function_code][0]
            if function_code.startswith("DP"):
                func = getattr(self.dp, function_name)
                return func(data)
            elif function_code.startswith("ML"):
                func = getattr(self.ml, function_name)
                return func(data)
            elif function_code.startswith("DL"):
                func = getattr(self.dl, function_name)
                return func(data)
            elif function_code.startswith("AUTO"):
                func = getattr(self.aml, function_name)
                return func(data)
            elif function_code.startswith("A_DL"):
                func = getattr(self.adl, function_name)
                return func(data)

        # Keeping this for user developement purposes
        elif function_code == self.fc.DataInput:
            return self.dp.DataInput(data)
        elif function_code == self.fc.HandleMissingData:
            return self.dp.HandleMissingData(data)
        elif function_code == self.fc.DataEncoding:
            return self.dp.DataEncoding(data)
        elif function_code == self.fc.DataSplitting:
            return self.dp.DataSplitting(data)
        elif function_code == self.fc.FeatureScaleData:
            return self.dp.FeatureScaling(data)
        elif function_code == self.fc.DropColumns:
            return self.dp.DropColumns(data)
        elif function_code == self.fc.TimeStepData:
            return self.dp.TimeStepData(data)

        elif function_code == self.fc.SimpleLinearRegression:
            return self.ml.SimpleLinearRegression(data)
        elif function_code == self.fc.SimpleLinearRegressionPredict:
            return self.ml.SimpleLinearRegressionPredict(data)
        elif function_code == self.fc.MultipleLinearRegression:
            return self.ml.SimpleLinearRegression(data)
        elif function_code == self.fc.MultipleLinearRegressionPredict:
            return self.ml.MultipleLinearRegressionPredict(data)
        elif function_code == self.fc.PolynomialLinearRegression:
            return self.ml.PolynomialLinearRegression(data)
        elif function_code == self.fc.PolynomialLinearRegressionPredict:
            return self.ml.PolynomialLinearRegressionPredict(data)
        elif function_code == self.fc.SupportVectorRegression:
            return self.ml.SupportVectorRegression(data)
        elif function_code == self.fc.SupportVectorRegressionPredict:
            return self.ml.SupportVectorRegressionPredict(data)
        elif function_code == self.fc.DecisionTreeRegression:
            return self.ml.DecisionTreeRegression(data)
        elif function_code == self.fc.DecisionTreeRegressionPredict:
            return self.ml.DecisionTreeRegressionPredict(data)
        elif function_code == self.fc.RandomForestRegression:
            return self.ml.RandomForestRegression(data)
        elif function_code == self.fc.RandomForestRegressionPredict:
            return self.ml.RandomForestRegressionPredict(data)
        elif function_code == self.fc.LogisticRegression:
            return self.ml.LogisticRegression(data)
        elif function_code == self.fc.LogisticRegressionPredict:
            return self.ml.LogisticRegressionPredict(data)
        elif function_code == self.fc.KNearestNeighbors:
            return self.ml.KNearestNeighbors(data)
        elif function_code == self.fc.KNearestNeighborsPredict:
            return self.ml.KNearestNeighborsPredict(data)
        elif function_code == self.fc.SupportVectorMachine:
            return self.ml.SupportVectorMachine(data)
        elif function_code == self.fc.SupportVectorMachinePredict:
            return self.ml.SupportVectorMachinePredict(data)
        elif function_code == self.fc.KernelSupportVectorMachine:
            return self.ml.KernelSupportVectorMachine(data)
        elif function_code == self.fc.KernelSupportVectorMachinePredict:
            return self.ml.KernelSupportVectorMachinePredict(data)
        elif function_code == self.fc.NaiveBayes:
            return self.ml.NaiveBayes(data)
        elif function_code == self.fc.NaiveBayesPredict:
            return self.ml.NaiveBayesPredict(data)
        elif function_code == self.fc.DecisionTreeClassification:
            return self.ml.DecisionTreeClassification(data)
        elif function_code == self.fc.DecisionTreeClassificationPredict:
            return self.ml.DecisionTreeClassificationPredict(data)
        elif function_code == self.fc.RandomForestClassification:
            return self.ml.RandomForestClassification(data)
        elif function_code == self.fc.RandomForestClassificationPredict:
            return self.ml.RandomForestClassificationPredict(data)
        elif function_code == self.fc.HierarchicalClustering:
            return self.ml.HierarchicalClustering(data)
        elif function_code == self.fc.HierarchicalClusteringPredict:
            return self.ml.HierarchicalClusteringPredict(data)
        elif function_code == self.fc.KMeansClustering:
            return self.ml.KMeansClustering(data)
        elif function_code == self.fc.KMeansClusteringPredict:
            return self.ml.KMeansClusteringPredict(data)
        elif function_code == self.fc.XGBoost:
            return self.ml.XGBoost(data)
        elif function_code == self.fc.XGBoostPredict:
            return self.ml.XGBoostPredict(data)

        elif function_code == self.fc.ArtificialNeuralNetworkInit:
            return self.dl.ArtificialNeuralNetworkInit(data)
        elif function_code == self.fc.ArtificialNeuralNetworkTrain:
            return self.dl.ArtificialNeuralNetworkTrain(data)
        elif function_code == self.fc.ArtificialNeuralNetworkEvaluate:
            return self.dl.ArtificialNeuralNetworkEvaluate(data)
        elif function_code == self.fc.ArtificialNeuralNetworkPredict:
            return self.dl.ArtificialNeuralNetworkPredict(data)
        elif function_code == self.fc.SelfOrganizingMapsInit:
            return self.dl.SelfOrganizingMapsInit(data)
        elif function_code == self.fc.SelfOrganizingMapsTrain:
            return self.dl.SelfOrganizingMapsTrain(data)
        elif function_code == self.fc.SelfOrganizingMapsPredict:
            return self.dl.SelfOrganizingMapsPredict(data)
        elif function_code == self.fc.RestrictedBoltzmannMachineInit:
            return self.dl.RestrictedBoltzmannMachineInit(data)
        elif function_code == self.fc.RestrictedBoltzmannMachineTrain:
            return self.dl.RestrictedBoltzmannMachineTrain(data)
        elif function_code == self.fc.RestrictedBoltzmannMachineTransform:
            return self.dl.RestrictedBoltzmannMachineTransform(data)



from setuptools import setup, find_packages

setup(
    name="autogonai",
    version="0.2.0",
    description="Python connector for Autogon Public APIs",
    author="David Mbatuegwu",
    author_email="david@autogon.ai",
    maintainer="Chiemezie Njoku",
    maintainer_email="manuel@autogon.ai",
    url="https://github.com/autogoninc/autogonai-python",
    packages=find_packages(),
    install_requires=["python-dotenv==1.0.0"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)



# Import necessary libraries
from autogonai.development.base import BaseBlock, gen_params


# Define class AutoRegression
class AutoRegression(BaseBlock):
    def set_params(
        self,
        model_name: str = None,
        time_left: int = None,
        run_time_limit: int = None,
        n_jobs: int = None,
    ):
        """
        Sets parameters for AutoRegression class.

        Args:
            model_name (str, optional): Name of the regression model. Defaults to None.
            time_left (int, optional): Time left for the regression. Defaults to None.
            run_time_limit (int, optional): Time limit for the regression run. Defaults to None.
            n_jobs (int, optional): Number of jobs for the regression. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoRegressionPredict
class AutoRegressionPredict(BaseBlock):
    def set_params(self, model_name: str = None):
        """
        Sets parameters for AutoRegressionPredict class.

        Args:
            model_name (str, optional): Name of the regression model. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoRegressionMetrics
class AutoRegressionMetrics(BaseBlock):
    def set_params(self, model_name: str = None, metric: str = None):
        """
        Sets parameters for AutoRegressionMetrics class.

        Args:
            model_name (str, optional): Name of the regression model. Defaults to None.
            metric (str, optional): Metric for regression evaluation. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoClassification
class AutoClassification(BaseBlock):
    def set_params(
        self,
        model_name: str = None,
        time_left: int = None,
        run_time_limit: int = None,
        n_jobs: int = None,
    ):
        """
        Sets parameters for AutoClassification class.

        Args:
            model_name (str, optional): Name of the classification model. Defaults to None.
            time_left (int, optional): Time left for the classification. Defaults to None.
            run_time_limit (int, optional): Time limit for the classification run. Defaults to None.
            n_jobs (int, optional): Number of jobs for the classification. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoClassificationPredict
class AutoClassificationPredict(BaseBlock):
    def set_params(self, model_name: str = None):
        """
        Sets parameters for AutoClassificationPredict class.

        Args:
            model_name (str, optional): Name of the classification model. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params


# Define class AutoClassificationMetrics
class AutoClassificationMetrics(BaseBlock):
    def set_params(self, model_name: str = None, metric: str = None):
        """
        Sets parameters for AutoClassificationMetrics class.

        Args:
            model_name (str, optional): Name of the classification model. Defaults to None.
            metric (str, optional): Metric for classification evaluation. Defaults to None.

        Returns:
            dict: Parameters dictionary.
        """
        self.params = gen_params(locals())
        return self.params
